package cn.myhug.adp.base

import android.os.Bundle
import android.support.annotation.CallSuper
import android.support.annotation.CheckResult
import android.support.v4.app.FragmentActivity
import cn.myhug.adp.framework.MessageManager
import cn.myhug.adp.framework.UniqueIdGenerator
import cn.myhug.adp.framework.listener.MessageListener
import cn.myhug.adp.framework.listener.NetMessageListener
import com.trello.rxlifecycle2.LifecycleProvider
import com.trello.rxlifecycle2.LifecycleTransformer
import com.trello.rxlifecycle2.RxLifecycle
import com.trello.rxlifecycle2.android.ActivityEvent
import com.trello.rxlifecycle2.android.RxLifecycleAndroid
import io.reactivex.Observable
import io.reactivex.subjects.BehaviorSubject

open class BdBaseActivity : FragmentActivity(), LifecycleProvider<ActivityEvent> {
    override fun <T : Any?> bindToLifecycle(): LifecycleTransformer<T> {
        return RxLifecycleAndroid.bindActivity(lifecycleSubject)
    }

    private val lifecycleSubject = BehaviorSubject.create<ActivityEvent>()

    override fun lifecycle(): Observable<ActivityEvent> {
        return lifecycleSubject.hide()
    }

    @CheckResult
    override fun <T> bindUntilEvent(event: ActivityEvent): LifecycleTransformer<T> {
        return RxLifecycle.bindUntilEvent(lifecycleSubject, event)
    }

    val uniqueId = UniqueIdGenerator.getInstance().id

    fun registerListener(listener: NetMessageListener?) {
        if (listener != null && listener.tag == 0) {
            listener.tag = uniqueId
        }
        MessageManager.getInstance().registerListener(listener)
    }

    fun registerListener(cmd: Int, listener: NetMessageListener?) {
        if (listener != null && listener.tag == 0) {
            listener.tag = uniqueId
        }
        MessageManager.getInstance().registerListener(cmd, listener)
    }

    fun registerListener(listener: MessageListener<*>?) {
        if (listener != null && listener.tag == 0) {
            listener.tag = uniqueId
        }
        MessageManager.getInstance().registerListener(listener)
    }

    fun registerListener(cmd: Int, listener: MessageListener<*>?) {
        if (listener != null && listener.tag == 0) {
            listener.tag = uniqueId
        }
        MessageManager.getInstance().registerListener(cmd, listener)
    }

    fun unregisterListener(listener: MessageListener<*>?) {
        if (listener != null && listener.tag == 0) {
            listener.tag = uniqueId
        }
        MessageManager.getInstance().unRegisterListener(listener)
    }


    @CallSuper
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        lifecycleSubject.onNext(ActivityEvent.CREATE)
    }

    @CallSuper
    override fun onStart() {
        super.onStart()
        lifecycleSubject.onNext(ActivityEvent.START)
    }

    @CallSuper
    override fun onResume() {
        super.onResume()
        lifecycleSubject.onNext(ActivityEvent.RESUME)
    }

    @CallSuper
    override fun onPause() {
        lifecycleSubject.onNext(ActivityEvent.PAUSE)
        super.onPause()
    }

    @CallSuper
    override fun onStop() {
        lifecycleSubject.onNext(ActivityEvent.STOP)
        super.onStop()
    }


    @CallSuper
    override fun onDestroy() {
        lifecycleSubject.onNext(ActivityEvent.DESTROY)
        super.onDestroy()
        if (uniqueId != 0) {
            MessageManager.getInstance().unRegisterListener(uniqueId)
            MessageManager.getInstance().removeMessage(uniqueId)
        }
    }


}
