package cn.myhug.adp.lib.util

import android.app.Activity
import android.app.KeyguardManager
import android.content.ActivityNotFoundException
import android.content.ClipboardManager
import android.content.ComponentName
import android.content.Context
import android.content.Intent
import android.content.pm.PackageManager
import android.content.res.Resources
import android.graphics.Bitmap
import android.graphics.Color
import android.graphics.Matrix
import android.graphics.Paint
import android.graphics.Rect
import android.hardware.Camera
import android.location.LocationManager
import android.media.ExifInterface
import android.media.MediaScannerConnection
import android.net.Uri
import android.os.Build
import android.os.Handler
import android.os.Looper
import android.os.PowerManager
import android.os.Vibrator
import android.provider.MediaStore
import android.support.v4.app.Fragment
import android.support.v4.content.FileProvider
import android.text.TextPaint
import android.text.TextUtils
import android.util.DisplayMetrics
import android.util.TypedValue
import android.view.Gravity
import android.view.TouchDelegate
import android.view.View
import android.view.inputmethod.InputMethodManager
import android.widget.EditText
import android.widget.TextView
import android.widget.Toast

import com.nostra13.universalimageloader.core.ImageLoader
import com.nostra13.universalimageloader.core.assist.ImageSize

import java.io.File
import java.io.IOException
import java.lang.reflect.Field
import java.util.regex.Pattern

import cn.myhug.adp.R
import cn.myhug.adp.base.BdBaseActivity
import cn.myhug.adp.base.BdBaseApplication
import cn.myhug.adp.lib.util.BdNetUtil.NetworkStateInfo
import io.reactivex.Observable
import io.reactivex.android.schedulers.AndroidSchedulers
import io.reactivex.schedulers.Schedulers


class BdUtilHelper {


    companion object {
        const val XIAOMI_MARKET_PKG = "com.xiaomi.market"

        const val GOOGLE_PLAY_PKG = "com.android.vending"

        private val PERMISSION_AUDIO_RECORD = "android.permission.RECORD_AUDIO"

        private var displayMetricsDensity: Float = 0.toFloat()

        internal var displayMetricsWidthPixels: Int = 0

        internal var displayMetricsHeightPixels: Int = 0

        internal var deviceDataInited = false

        /* begin 获取两个经纬度间的距离，单位公里 */
        private val EARTH_RADIUS = 6378.137// 地球半径

        private var wakeLock: PowerManager.WakeLock? = null

        /**
         * 全局只保留一个Toast对象，要不然多个Toast会叠在一起
         */
        var mToast: Toast?= null

        private var sUmengChannel: String? = null


        @JvmStatic
        fun isXiaoMi(): Boolean {
            val temp = android.os.Build.MANUFACTURER
            val XIAOMI = "Xiaomi"
            if (XIAOMI.compareTo(temp, ignoreCase = true) != 0) {
                return false
            }
            val intent = Intent()
            intent.action = Intent.ACTION_VIEW
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            intent.data = Uri.parse("market://details?id=" + BdBaseApplication.getInst().packageName)
            val pManager = BdBaseApplication.getInst().packageManager
            val mApps = pManager.queryIntentActivities(intent, PackageManager.COMPONENT_ENABLED_STATE_DEFAULT)
            for (info in mApps) {
                if (XIAOMI_MARKET_PKG == info.activityInfo.packageName) {
                    intent.setPackage(info.activityInfo.packageName)
                    return true
                }
            }
            return false
        }


        @JvmStatic
        fun getUmengChannel(): String? {
            if (null == sUmengChannel) {
                try {
                    val info = BdBaseApplication.getInst().packageManager
                            .getApplicationInfo(BdBaseApplication.getInst().packageName,
                                    PackageManager.GET_META_DATA)
                    sUmengChannel = info.metaData.getString("UMENG_CHANNEL")
                } catch (e: PackageManager.NameNotFoundException) {

                }

            }
            return sUmengChannel
        }


        @JvmStatic
        fun isGooglPlayChannel(): Boolean {
            var ret = false
            try {
                val info = BdBaseApplication.getInst().packageManager
                        .getApplicationInfo(BdBaseApplication.getInst().packageName,
                                PackageManager.GET_META_DATA)
                val msg = info.metaData.getString("UMENG_CHANNEL")
                if ("google_play" == msg) {
                    ret = true
                }
            } catch (e: PackageManager.NameNotFoundException) {

            }

            return ret
        }

        @JvmStatic
        fun initDeviceData(context: Context) {
            var displayMetrics: DisplayMetrics = context.resources.displayMetrics
            if (context.resources != null) {
                displayMetricsDensity = displayMetrics.density
                displayMetricsWidthPixels = displayMetrics.widthPixels
                displayMetricsHeightPixels = displayMetrics.heightPixels
            }
            deviceDataInited = true
        }


        @JvmStatic
        fun getEquipmentWidth(context: Context): Int {
            if (!deviceDataInited) {
                initDeviceData(context)
            }

            return displayMetricsWidthPixels
        }


        @JvmStatic
        fun getEquipmentHeight(context: Context): Int {
            if (!deviceDataInited) {
                initDeviceData(context)
            }

            return displayMetricsHeightPixels
        }

        @JvmStatic
        fun dip2px(context: Context, dipValue: Float): Int {
            if (!deviceDataInited) {
                initDeviceData(context)
            }

            return (dipValue * displayMetricsDensity + 0.5f).toInt()
        }


        @JvmStatic
        fun sp2px(context: Context, spValue: Float): Int {
            val fontScale = context.resources.displayMetrics.scaledDensity
            return (spValue * fontScale + 0.5f).toInt()
        }


        @JvmStatic
        fun px2dip(context: Context, pxValue: Float): Int {
            if (!deviceDataInited) {
                initDeviceData(context)
            }

            return (pxValue / displayMetricsDensity + 0.5f).toInt()
        }


        @JvmStatic
        fun getEquipmentDensity(context: Context): Float {
            if (!deviceDataInited) {
                initDeviceData(context)
            }

            return displayMetricsDensity
        }


        /**
         * 显示一个Toast，可自定义时间
         *
         * @param context
         * @param str
         * @param duration
         */
        @JvmStatic
        fun showToast(context: Context?, str: String?) {
            if (TextUtils.isEmpty(str) || context == null) {
                return
            }


            if ( context is BdBaseActivity ){
                Observable.just(str).subscribeOn(Schedulers.io()).observeOn(AndroidSchedulers.mainThread())
                        .compose(context.bindToLifecycle()).subscribe {
                            if (mToast == null) {
                                mToast = Toast.makeText(context, str, Toast.LENGTH_SHORT)
                                mToast!!.view.setBackgroundResource(R.drawable.custom_toast_bg)

                                val textView = mToast!!.view.findViewById<TextView>(android.R.id.message)
                                textView.setTextColor(Color.WHITE)
                                val padding = context.resources.getDimensionPixelOffset(R.dimen.default_gap_30)
                                textView.setPadding(padding, 0, padding, 0)
                                mToast!!.setGravity(Gravity.CENTER, 0, dip2px(context, 100f))

                            } else {
                                mToast!!.duration = Toast.LENGTH_SHORT
                                mToast!!.setText(str)
                            }

                            mToast!!.show()
                        }
            } else {
                if (mToast == null) {
                    mToast = Toast.makeText(context, str, Toast.LENGTH_SHORT)
                    mToast!!.view.setBackgroundResource(R.drawable.custom_toast_bg)

                    val textView = mToast!!.view.findViewById<TextView>(android.R.id.message)
                    textView.setTextColor(Color.WHITE)
                    val padding = context.resources.getDimensionPixelOffset(R.dimen.default_gap_30)
                    textView.setPadding(padding, 0, padding, 0)
                    mToast!!.setGravity(Gravity.CENTER, 0, dip2px(context, 100f))

                } else {
                    mToast!!.duration = Toast.LENGTH_SHORT
                    mToast!!.setText(str)
                }

                mToast!!.show()
            }



        }


        /**
         * 判断当前网络是否可用
         *
         * @return
         */

        @JvmStatic
        fun isNetOk(): Boolean {
            return BdNetUtil.getStatusInfo() != NetworkStateInfo.UNAVAIL
        }


        @JvmStatic
        fun showToast(context: Context?, stringId: Int) {
            if ( context == null ){
                return
            }

            val str = context.resources.getString(stringId)
            showToast(context, str)
        }


        @JvmStatic
        fun hideSoftKeyPad(context: Context, view: View?) {
            try {
                if (view == null) {
                    return
                } else if (view.windowToken == null) {
                    return
                }

                val inputManager = context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
                inputManager.hideSoftInputFromWindow(view.windowToken, InputMethodManager.HIDE_NOT_ALWAYS)
            } catch (ex: Throwable) {
                BdLog.e("UtilHelper", "hideSoftKeyPad", "error = " + ex.message)
            }

        }


        @JvmStatic
        fun showSoftKeyPadDelay(context: Context, view: EditText) {
            Handler().postDelayed({ showSoftKeyPad(context, view) }, 50)
        }


        @JvmStatic
        fun showSoftKeyPadDelay(context: Context, view: EditText, keyboardDelay: Int) {
            Handler().postDelayed({ showSoftKeyPad(context, view) }, keyboardDelay.toLong())
        }


        @JvmStatic
        fun showSoftKeyPad(context: Context, view: EditText) {
            try {
                view.isFocusableInTouchMode = true
                view.requestFocus()
                val inputManager = context.getSystemService(Context.INPUT_METHOD_SERVICE) as InputMethodManager
                inputManager.showSoftInput(view, InputMethodManager.RESULT_UNCHANGED_SHOWN)
            } catch (ex: Throwable) {
                BdLog.e("UtilHelper", "showSoftKeyPad", "error = " + ex.message)
            }

        }




        /**
         * 获取一个对象隐藏的属性，并设置属性为public属性允许直接访问�?
         *
         * @return [Field] 如果无法读取，返回null；返回的Field需要使用者自己缓存，本方法不做缓存�?
         */
        @JvmStatic
        fun getDeclaredField(`object`: Any, field_name: String): Field? {
            var cla: Class<*> = `object`.javaClass
            var field: Field? = null
            while (cla != Any::class.java) {
                try {
                    field = cla.getDeclaredField(field_name)
                    field!!.isAccessible = true
                    return field
                } catch (e: Exception) {

                }

                cla = cla.superclass
            }
            return null
        }


        @JvmStatic
        fun isGif(data: ByteArray): Boolean {
            var isGif = false
            try {
                if (data[0] == 'G'.toByte() && data[1] == 'I'.toByte() && data[2] == 'F'.toByte()) {
                    isGif = true
                }
            } catch (ex: Exception) {
                isGif = false
            }

            return isGif
        }


        @JvmStatic
        fun getScreenSize(activity: Activity): DisplayMetrics? {
            var result: DisplayMetrics? = null
            try {
                result = DisplayMetrics()
                activity.windowManager.defaultDisplay.getMetrics(result)
            } catch (e: Exception) {
                BdLog.i("UtilHelper", "getScreenSize", e.toString())
            }

            return result
        }

        // 1. 粗略计算文字宽度
        @JvmStatic
        fun measureTextWidth(paint: Paint?, str: String?): Float {
            return if (paint == null || str == null) {
                0f
            } else paint.measureText(str)
        }

        // 2. 计算文字所在矩形，可以得到宽高
        @JvmStatic
        fun measureText(paint: Paint, str: String): Rect {
            val rect = Rect()
            paint.getTextBounds(str, 0, str.length, rect)
            return rect
        }

        // 3. 精确计算文字宽度
        @JvmStatic
        fun getTextWidth(paint: Paint, str: String?): Int {
            var iRet = 0
            if (str != null && str.length > 0) {
                val len = str.length
                val widths = FloatArray(len)
                paint.getTextWidths(str, widths)
                for (j in 0 until len) {
                    iRet += Math.ceil(widths[j].toDouble()).toInt()
                }
            }
            return iRet
        }


        @JvmStatic
        fun setTextSize(c: Context?, paint: TextPaint, size: Float): TextPaint {
            val r: Resources?
            if (c == null) {
                r = Resources.getSystem()
            } else {
                r = c.resources
            }
            if (r != null) {
                paint.textSize = TypedValue.applyDimension(TypedValue.COMPLEX_UNIT_SP, size, r.displayMetrics)
            }
            return paint
        }

        @JvmStatic
        fun getFontHeight(context: Context, fontSize: Float): Int {
            val paint = TextPaint()
            setTextSize(context, paint, fontSize)
            val fm = paint.fontMetrics
            return Math.ceil((fm.descent - fm.ascent).toDouble()).toInt()
        }

        @JvmStatic
        fun setWindowAlpha(activity: Activity, alpha: Float) {
            val params = activity.window.attributes
            // params.alpha = alpha;
            params.screenBrightness = alpha
            activity.window.attributes = params
        }




        @JvmStatic
        private fun rad(d: Double): Double {
            return d * Math.PI / 180.0
        }


        /* end 获取两个经纬度间的距离，单位公里 */

        // 获取两个经纬度间的距离，单位公里
        @JvmStatic
        fun getDistance(lat1: Double, lng1: Double, lat2: Double, lng2: Double): Double {
            val radLat1 = rad(lat1)
            val radLat2 = rad(lat2)
            val a = radLat1 - radLat2
            val b = rad(lng1) - rad(lng2)

            var s = 2 * Math.asin(Math.sqrt(Math.pow(Math.sin(a / 2), 2.0) + (Math.cos(radLat1) * Math.cos(radLat2)
                    * Math.pow(Math.sin(b / 2), 2.0))))
            s = s * EARTH_RADIUS
            s = (Math.round(s * 10000) / 10000).toDouble()
            return s
        }

        // 检测客户端是否安装某个app
        @JvmStatic
        fun hasInstallApp(context: Context, packageName: String?): Boolean {
            if (packageName == null || packageName.length == 0) {
                return false
            }

            val packageManager = context.packageManager// 获取packagemanager
            val pinfo = packageManager.getInstalledPackages(0)// 获取所有已安装程序的包信息
            if (pinfo != null) {
                for (i in pinfo.indices) {
                    val pn = pinfo[i].packageName
                    if (pn == packageName) {
                        return true
                    }
                }
            }
            return false
        }



        // 检查是否有快捷方式
        @JvmStatic
        fun checkShortCut(context: Context, appName: String): Boolean {
            var hasShortCut = false
            try {
                val cr = context.contentResolver
                val AUTHORITY1 = "com.android.launcher.settings"
                val AUTHORITY2 = "com.android.launcher2.settings"
                var contentUri = ""
                if (android.os.Build.VERSION.SDK_INT < 8) {
                    contentUri = "content://$AUTHORITY1/favorites?notify=true"
                } else {
                    contentUri = "content://$AUTHORITY2/favorites?notify=true"
                }
                val CONTENT_URI = Uri.parse(contentUri)
                val c = cr.query(CONTENT_URI, arrayOf("title", "iconResource"), "title=?", arrayOf(appName), null)
                if (c != null && c.count > 0) {
                    hasShortCut = true
                }
                c!!.close()
            } catch (e: Exception) {
                BdLog.i("UtilHelper", "checkShortCut", e.toString())
            }

            return hasShortCut
        }


        /**
         * 检查是否为主线程
         *
         * @throws Throwable
         */
        @JvmStatic
        fun checkMainThread() {
            if (BdBaseApplication.isDebugMode()) {
                var error = false
                if (Looper.myLooper() == null || Looper.getMainLooper() != Looper.myLooper()) {
                    error = true
                }

                if (error == true) {
                    val buidler = StringBuilder(100)
                    val elements = Thread.currentThread().stackTrace
                    for (i in 1 until elements.size) {
                        buidler.append(elements[i].className)
                        buidler.append(".")
                        buidler.append(elements[i].methodName)
                        buidler.append("<-")
                    }
                    throw Error("can not be called on none-main thread! trace = " + buidler.toString())
                }
            }
        }

        /**
         * 扩大view的点击区域，不会超过父view
         *
         * @param context
         * @param view
         * @param l
         * @param t
         * @param r
         * @param b
         */
        @JvmStatic
        fun addToParentArea(context: Context, view: View, l: Int, t: Int, r: Int, b: Int) {

            val pl = dip2px(context, l.toFloat())
            val pt = dip2px(context, t.toFloat())
            val pr = dip2px(context, r.toFloat())
            val pb = dip2px(context, b.toFloat())

            val parent = view.parent as View
            parent.post {
                val rect = Rect()
                view.getHitRect(rect)
                rect.right += pr
                rect.left -= pl
                rect.bottom += pb
                rect.top -= pt
                parent.touchDelegate = TouchDelegate(rect, view)
            }
        }


        @JvmStatic
        fun getBitmapFromURI(context: Context, contentUri: Uri): Bitmap {
            return ImageLoader.getInstance().loadImageSync(contentUri.toString(), ImageSize(400, 400))

        }



        @JvmStatic
        fun printStack() {
            val e = Exception("log===")
            e.printStackTrace()
        }


        @JvmStatic
        fun printStack(flag: String) {
            val e = Exception("$flag===")
            e.printStackTrace()
        }


        @JvmStatic
        fun hasMarketApp(pkg: String?): Boolean {
            if (null == pkg || "" == pkg.trim { it <= ' ' }) {
                return false
            }

            val intent = Intent()
            intent.action = Intent.ACTION_VIEW
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            intent.data = Uri.parse("market://details?id=" + BdBaseApplication.getInst().packageName)
            val pManager = BdBaseApplication.getInst().packageManager
            val mApps = pManager.queryIntentActivities(intent, PackageManager.COMPONENT_ENABLED_STATE_DEFAULT)
            for (info in mApps) {
                if (pkg == info.activityInfo.packageName) {
                    intent.setPackage(info.activityInfo.packageName)
                    return true
                }
            }
            return false
        }


        @JvmStatic
        fun openMarket(pkg: String): Boolean {
            try {
                val intent = Intent()
                intent.action = Intent.ACTION_VIEW
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                //com.android.chrome
                intent.data = Uri.parse("market://details?id=" + BdBaseApplication.getInst().packageName)
                val pManager = BdBaseApplication.getInst().packageManager
                val mApps = pManager.queryIntentActivities(intent, PackageManager.COMPONENT_ENABLED_STATE_DEFAULT)
                for (info in mApps) {
                    if (pkg == info.activityInfo.packageName) {
                        intent.setPackage(info.activityInfo.packageName)
                        break
                    }
                }
                BdBaseApplication.getInst().startActivity(intent)
                return true
            } catch (e: Exception) {
                e.printStackTrace()
            }

            return false
        }


        @JvmStatic
        fun openUrl(context: Context, url: String): Boolean {
            try {
                val uri = Uri.parse(url) //url为你要链接的地址
                val intent = Intent(Intent.ACTION_VIEW, uri)
                context.startActivity(intent)
                return true
            } catch (e: Exception) {
                e.printStackTrace()
            }

            return false
        }


        @JvmStatic
        fun vibator(mils: Int) {
            val vibrator = BdBaseApplication.getInst().getSystemService(Context.VIBRATOR_SERVICE) as Vibrator
            vibrator.vibrate(mils.toLong())
        }


        @JvmStatic
        fun takePhoto(activity: Activity, requestCode: Int) {
            try {
                val intent = Intent(Intent.ACTION_PICK,
                        android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI)
                activity.startActivityForResult(intent, requestCode)
            } catch (e: ActivityNotFoundException) {

            }

        }


        @JvmStatic
        fun takePhoto(fragment: android.support.v4.app.Fragment, requestCode: Int) {
            try {
                val intent = Intent(Intent.ACTION_PICK,
                        android.provider.MediaStore.Images.Media.EXTERNAL_CONTENT_URI)
                fragment.startActivityForResult(intent, requestCode)
            } catch (e: ActivityNotFoundException) {

            }

        }


        @JvmStatic
        fun showCamera(activity: Activity, requestCode: Int, path: String) {
            val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE)
            val file = BdFileHelper.getFile(path) ?: return

            val uri = Uri.fromFile(file)
            intent.putExtra(MediaStore.EXTRA_OUTPUT, uri)
            activity.startActivityForResult(intent, requestCode)
        }


        @JvmStatic
        fun showFrontCamera(activity: Activity, requestCode: Int, path: String) {
            val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE)
            intent.putExtra("camerasensortype", 2) // 调用前置摄像头
            intent.putExtra("android.intent.extras.CAMERA_FACING", 1)
            intent.putExtra("autofocus", true) // 自动对焦
            val file = BdFileHelper.getFile(path) ?: return

            val uri = Uri.fromFile(file)
            intent.putExtra(MediaStore.EXTRA_OUTPUT, uri)
            activity.startActivityForResult(intent, requestCode)
        }


        @JvmStatic
        fun getFrontCamera(context: Context, path: String): Intent? {
            val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE)

            intent.putExtra("camerasensortype", 2) // 调用前置摄像头
            intent.putExtra("android.intent.extras.CAMERA_FACING", 1)
            intent.putExtra("autofocus", true) // 自动对焦
            val file = BdFileHelper.getFile(path) ?: return null

            val uri: Uri

            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
                intent.flags = Intent.FLAG_GRANT_READ_URI_PERMISSION
                uri = FileProvider.getUriForFile(context, context.applicationContext.packageName+".fileprovider", file)
            } else {
                uri = Uri.fromFile(file)
            }

            intent.putExtra(MediaStore.EXTRA_OUTPUT, uri)

            return intent
        }


        @JvmStatic
        fun showFrontCamera(fragment: Fragment, requestCode: Int, path: String) {
            val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE)
            intent.putExtra("camerasensortype", 2) // 调用前置摄像头
            intent.putExtra("android.intent.extras.CAMERA_FACING", Camera.CameraInfo.CAMERA_FACING_FRONT)
            intent.putExtra("autofocus", true) // 自动对焦
            val file = BdFileHelper.getFile(path)
            val uri = Uri.fromFile(file)
            intent.putExtra(MediaStore.EXTRA_OUTPUT, uri)
            fragment.startActivityForResult(intent, requestCode)
        }


        @JvmStatic
        fun showCamera(fragment: android.support.v4.app.Fragment, requestCode: Int, path: String) {
            val intent = Intent(MediaStore.ACTION_IMAGE_CAPTURE)
            val file = BdFileHelper.getFile(path)
            val uri = Uri.fromFile(file)
            intent.putExtra(MediaStore.EXTRA_OUTPUT, uri)
            fragment.startActivityForResult(intent, requestCode)
        }

        @JvmStatic
        fun getPhotoPath(data: Intent): String? {
            var picturePath: String? = null
            try {
                val selectedImage = data.data
                val filePathColumns = arrayOf(MediaStore.Images.Media.DATA)
                val c = BdBaseApplication.getInst().contentResolver.query(selectedImage!!, filePathColumns, null, null, null)
                c!!.moveToFirst()
                val columnIndex = c.getColumnIndex(filePathColumns[0])
                picturePath = c.getString(columnIndex)
                c.close()
            } catch (e: Exception) {
                e.printStackTrace()
            }

            return picturePath
        }


        @JvmStatic
        fun startGPSSettingActivity(context: Context) {
            try {
                context.startActivity(Intent(android.provider.Settings.ACTION_LOCATION_SOURCE_SETTINGS))
            } catch (e: Exception) {

            }

        }


        @JvmStatic
        fun checkGPSState(): Boolean {
            try {
                val locationManager = BdBaseApplication.getInst().getSystemService(Context.LOCATION_SERVICE) as LocationManager
                // 通过GPS卫星定位，定位级别可以精确到街（通过24颗卫星定位，在室外和空旷的地方定位准确、速度快）
                val gps = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)
                // 通过WLAN或移动网络(3G/2G)确定的位置（也称作AGPS，辅助GPS定位。主要用于在室内或遮盖物（建筑群或茂密的深林等）密集的地方定位）
                val network = locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)
                return if (gps || network) {
                    true
                } else false
            } catch (e: Exception) {
                e.printStackTrace()
            }

            return false

        }

        @JvmStatic
        fun insertToMedia(file: File?): Boolean {
            if (file == null) {
                return false
            }
            Thread(Runnable {
                try {
                    MediaStore.Images.Media.insertImage(BdBaseApplication.getInst().contentResolver, file.absolutePath, "baobao", file.name)
                    MediaScannerConnection.scanFile(BdBaseApplication.getInst(), arrayOf(file.absolutePath), null) { path, uri ->
                        BdUtilHelper.showToast(BdBaseApplication.getInst(),"已加入相册")
                    }

                } catch (e: Exception) {
                    e.printStackTrace()
                }
            }).start()
            return true
        }







        @JvmStatic
        fun checkAudio(): Boolean {
            return checkPermission(PERMISSION_AUDIO_RECORD)
        }


        @JvmStatic
        fun checkPermission(permission: String): Boolean {
            val pkgName = BdBaseApplication.getInst().packageName
            // 结果为0则表示使用了该权限，-1则表求没有使用该权限
            val reslut = BdBaseApplication.getInst().packageManager.checkPermission(permission, pkgName)
            return if (reslut == PackageManager.PERMISSION_GRANTED) {
                true
            } else {
                false
            }
        }


        @JvmStatic
        fun copy(content: String, context: Context) {
            // 得到剪贴板管理器
            val cmb = context.getSystemService(Context.CLIPBOARD_SERVICE) as ClipboardManager
            cmb.text = content.trim { it <= ' ' }
        }


        /**
         * 调起应用市场
         * @param context
         * @param packageName
         */
        @JvmStatic
        fun commentInAppMarket(context: Context, packageName: String) {
            try {
                val uri = Uri.parse("market://details?id=$packageName")
                val intent = Intent(Intent.ACTION_VIEW, uri)
                intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
                context.startActivity(intent)

            } catch (e: Exception) {
                Toast.makeText(context, "您没有安装应用市场", Toast.LENGTH_SHORT).show()
            }

        }


        @JvmStatic
        fun getRegexMatch(input: String, regex: String): String? {
            val pattern = Pattern.compile(regex, Pattern.CASE_INSENSITIVE)// 匹配的模式
            val m = pattern.matcher(input)
            while (m.find()) {
                return m.group(1)
            }

            return null
        }


        @JvmStatic
        fun readPictureDegree(uri: Uri?): Int {
            if (uri == null) {
                return 0
            }
            var degree = 0
            try {
                val path = getRealPathFromURI(uri)

                if (!StringHelper.checkString(path)) {
                    return degree
                }

                val exifInterface = ExifInterface(path)
                val orientation = exifInterface.getAttributeInt(ExifInterface.TAG_ORIENTATION, ExifInterface.ORIENTATION_NORMAL)
                when (orientation) {
                    ExifInterface.ORIENTATION_ROTATE_90 -> degree = 90
                    ExifInterface.ORIENTATION_ROTATE_180 -> degree = 180
                    ExifInterface.ORIENTATION_ROTATE_270 -> degree = 270
                }
            } catch (e: IOException) {
                e.printStackTrace()
            }

            return degree
        }


        @JvmStatic
        fun getRealPathFromURI(contentUri: Uri?): String? {
            if (contentUri == null) {
                return null
            }
            try {
                val proj = arrayOf(MediaStore.Images.Media.DATA)
                val cursor = BdBaseApplication.getInst().contentResolver.query(contentUri, proj, null, null, null)
                val column_index = cursor!!.getColumnIndexOrThrow(MediaStore.Images.Media.DATA)
                cursor.moveToFirst()
                val path = cursor.getString(column_index)
                cursor.close()
                return path
            } catch (e: Exception) {

            }

            return contentUri.path
        }


        @JvmStatic
        fun rotaingImageView(angle: Int, bitmap: Bitmap): Bitmap {
            //旋转图片 动作
            val matrix = Matrix()
            matrix.postRotate(angle.toFloat())
            println("angle2=$angle")
            // 创建新的图片
            return Bitmap.createBitmap(bitmap, 0, 0,
                    bitmap.width, bitmap.height, matrix, true)
        }
    }
}
