package cn.myhug.devlib.db

import android.content.Context
import cn.myhug.adp.base.BdBaseApplication
import cn.myhug.adp.lib.util.BdLog

import cn.myhug.devlib.utils.JsonUtil
import com.lusfold.androidkeyvaluestore.KVStore
import io.reactivex.functions.Consumer
import io.reactivex.internal.operators.observable.ObservableJust
import io.reactivex.schedulers.Schedulers

object KVStore {
    private const val DB_NAME = "KV_STORE";
    private val SP = BdBaseApplication.getInst().applicationContext.getSharedPreferences("kv_store3", Context.MODE_PRIVATE)

    init {
        com.lusfold.androidkeyvaluestore.KVStore.init(BdBaseApplication.getInst().applicationContext, DB_NAME)
    }
    fun putString(key: String, value: String?) {
        SP.edit().putString(key, value).apply()
        BdLog.i("put key=${key},value=${value}")
        ObservableJust.just(value).observeOn(Schedulers.io()).subscribe(Consumer {
            com.lusfold.androidkeyvaluestore.KVStore.getInstance().insertOrUpdate(key, value)
        })

    }

    fun putInt(key: String, value: Int=0) {
        SP.edit().putInt(key, value).apply()
        BdLog.i("put key=${key},value=${value}")
    }

    fun putObject(key: String, value: Any?) {
        BdLog.i("put key=${key},value=${value}")
        ObservableJust.just(value).observeOn(Schedulers.io()).subscribe {
            KVStore.getInstance().insertOrUpdate(key, value?.let { it -> JsonUtil.toJson(it) })
        }
    }

    fun getString(key: String, value: String?):String? {
        val ret= SP.getString(key,value)
        return ret
    }

    fun getInt(key: String, value: Int=0):Int {
        val ret= SP.getInt(key,value)
        return ret
    }

    inline fun<reified T:Any> getObject(key: String, value: T? = null):T? {
        try {
            if (!com.lusfold.androidkeyvaluestore.KVStore.getInstance().keyExists(key)) {
                return null
            }
            val v = com.lusfold.androidkeyvaluestore.KVStore.getInstance().get(key)
            return JsonUtil.convertJson(v, T::class.java)
        } catch (e: Exception) {
            e.printStackTrace()
        }
        return value
    }

}
