package cn.myhug.devlib.newnetwork

import cn.myhug.adp.lib.util.BdLog
import com.facebook.stetho.okhttp3.StethoInterceptor
import okhttp3.*
import retrofit2.CallAdapter
import retrofit2.Retrofit
import java.util.concurrent.TimeUnit

/**
 * Created by zhengxin on 2018/2/8.
 */

object RetrofitClient {
    private val dispatcher = Dispatcher()

    var httpclient: OkHttpClient
        private set
    var retrofit: Retrofit
        private set

    init {
        dispatcher.maxRequests = 30


        val builder = OkHttpClient().newBuilder()
        builder.readTimeout(15, TimeUnit.SECONDS)
        builder.connectTimeout(7, TimeUnit.SECONDS)
        builder.addInterceptor { chain ->
            val request = chain.request().newBuilder().addHeader("User-Agent", NetworkConfig.UA).build()

            var fullUrl = request.url().toString()
            if (request.body() is FormBody) {
                val body = request.body() as FormBody
                for (i in 0 until body.size()) {

                    if (fullUrl.contains("?")) {
                        fullUrl = fullUrl.plus("&")
                    } else {
                        fullUrl = fullUrl.plus("?")
                    }
                    fullUrl = fullUrl.plus(body.encodedName(i) + "=" + body.encodedValue(i))

                }
            }

            BdLog.i(fullUrl)
            chain.proceed(request)
        }
        builder.addInterceptor(StethoInterceptor())

        builder.dispatcher(dispatcher)
        httpclient = builder.build()
        this.retrofit = Retrofit.Builder()
                .baseUrl(NetworkConfig.HOST)
                .addConverterFactory(GsonSafeConverterFactory.create())
                .addCallAdapterFactory(RxJavaThreadDealCallAdapterFactory.create())
                .client(httpclient)
                .build()
    }

    fun addInterceptor(interceptor: Interceptor){
        val builder = httpclient.newBuilder()
        builder.readTimeout(30, TimeUnit.SECONDS)
        builder.connectTimeout(15, TimeUnit.SECONDS)
        builder.interceptors().add(0,interceptor)
        httpclient = builder.build()
        val retrofitBuilder = retrofit.newBuilder()
        retrofitBuilder.client(httpclient)
        retrofit = retrofitBuilder.build()
    }

    fun addCallAdapterFactory(factory: CallAdapter.Factory){
        val retrofitBuilder = retrofit.newBuilder()
        retrofitBuilder.addCallAdapterFactory(factory)
        retrofit = retrofitBuilder.build();
    }

    fun cancelAll() {
        dispatcher.cancelAll()
    }
}
